package com.jetbyte.cache4;

public interface Cache
{
   public Object find(String keyString);
   public Object find(Key cacheKey);

   public void add(String keyValue, Object obj);
   public void add(Key cacheKey, Object obj);
   
   public void remove(String keyValue);
   public void remove(Key cacheKey);
   
   public String name();
   public String type();

   public int entries();
   public int hits();
   public int misses();
   
   public void flush();

   public class Key implements Comparable
   {
      public Key()
      {
         setKey("");
      }

      public Key(String key)
      {
         keyValue = key;
         hashCode = keyValue.hashCode();
      }

      public void setKey(String key)
      {
         keyValue = key;
         hashCode = keyValue.hashCode();
      }

      public String toString()
      {
         return keyValue;
      }

      public int hashCode()
      {
         return hashCode;
      }

      public boolean equals(Object obj)
      {
         if (this == obj)
         {
            return true;
         }
         else if (obj == null || getClass() != obj.getClass())
         {
            return false;
         }

         Key rhs = (Key) obj;

         return keyValue.equals(rhs.keyValue);
      }

      public int compareTo(Object obj)
      {
         if (this.equals(obj))
         {
            return 0;
         }

         Key rhs = (Key) obj;

         return keyValue.compareTo(rhs.keyValue);
      }

      private String keyValue;

      private int hashCode;
   }

   public static class ObjectRef
   {
      protected ObjectRef(
         Cache cache, 
         Key key, 
         Object obj)
      {
         this.obj = obj;
         this.key = key;
         this.cache = cache;

         lastUsedTime = System.currentTimeMillis(); 
      }

      synchronized public Object getObject()
      {
         lastUsedTime = System.currentTimeMillis(); 

         return obj;
      }

      protected String getKey()
      {
         return key.toString();
      }

      synchronized protected long getLastUsedTime()
      {
         return lastUsedTime;
      }

      protected void removeFromCache()
      {
         cache.remove(key);
      }

      private Cache cache;
      private Key key;
      private Object obj;
      private long lastUsedTime = 0;
   }
}
